import os, csv
import numpy
from datetime import datetime

# main 
base_dir = os.getcwd()

# directory to adcirc wind data
adcirc_wind_dir = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Puget_Sound_ADCIRC_Results-From_ATKINS\ADCIRC_Model_Runs\Model Runs - ADCIRC - Event List and BCs\ADCIRC_BCs_Wind"

# directory to wis wave data
wis_wave_dir  = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Data\NorthWAOceanWaves\HINDCAST\WIS"

# open files to save time series of swell
swell_hs_file = (open("swell_hs.csv", 'wb'))
swell_hs_writer = csv.writer(swell_hs_file)
swell_tp_file = (open("swell_tp.csv", 'wb'))
swell_tp_writer = csv.writer(swell_tp_file)
swell_dir_file = (open("swell_dir.csv", 'wb'))
swell_dir_writer = csv.writer(swell_dir_file)

os.chdir(adcirc_wind_dir)
# load date of storm events 
event_list = numpy.genfromtxt('EventList.txt', dtype=None)
event_dates = []
for i, event in enumerate(event_list):
    string_date = event[0] + ' ' + event[1]
    event_dates.append(datetime.strptime(string_date, "%d-%b-%Y %H:%M:%S"))

os.chdir(wis_wave_dir)    
# load swell from WIS data, and retain swell only 
wis_wave_filename = "ST83001_v01.onlns"
wis_wave_data = numpy.loadtxt(wis_wave_filename)
wis_swell_data = []
for irec, rec in enumerate(wis_wave_data):
    string_date = str(int(rec[0]))
    date = datetime.strptime(string_date, "%Y%m%d%H%M%S")
    wis_swell_data.append([date, rec[25], rec[26], rec[31]])

os.chdir(base_dir)
post_date = datetime.strptime("31-Dec-1978 23:00:00", "%d-%b-%Y %H:%M:%S")
for storm_num in range(1,151):

    # ignore pre-1979 storms
    storm_date = event_dates[storm_num-1]
    tdiff = (storm_date-post_date).total_seconds()
    if(tdiff<=0): continue

    print "storm:", storm_num, event_dates[storm_num-1]   

    # append first data as storm_num
    swell_hs = []
    swell_hs.append(storm_num)
    swell_tp = []
    swell_tp.append(storm_num)
    swell_dir = []
    swell_dir.append(storm_num)

    # loop through swell data
    for irec, rec in enumerate(wis_swell_data):
        rec_date = rec[0] # change from GMT/UTC to PST time zone
        date_diff = (rec_date - storm_date).total_seconds()/3600 - 8
        # retrieve date 6 days before and 2 days after of the storm 
        if(date_diff>=-143 and date_diff<=48):
            print rec_date, rec[1], rec[2], rec[3]
            swell_hs.append(rec[1])
            swell_tp.append(rec[2])
            swell_dir.append(rec[3])
    if(len(swell_hs)==1):
        for i in range(192):
            swell_hs.append("nan")
            swell_tp.append("nan")
            swell_dir.append("nan")

    # write time series of wind for this storm
    swell_hs_writer.writerow(swell_hs)
    swell_tp_writer.writerow(swell_tp)
    swell_dir_writer.writerow(swell_dir)
    
os.chdir(base_dir)

swell_hs_file.close()
swell_tp_file.close()
swell_dir_file.close()