import os, csv
import numpy

# find col of each transect in TWL_look_up matrix 
def match_tsctid(TWL_matrix_header, tsct_info):

    tsct_map = []

    for tsct in tsct_info:
        tsct_id = int(tsct[0])
        for irec, rec in enumerate(TWL_matrix_header):
            if irec<=3: continue
            tsct_id_in_matrix = rec.split("_")[1]
            if(tsct_id==int(tsct_id_in_matrix)):
                tsct_map.append(irec)
                break
                
    return tsct_map
            
# look up TWL in TWL_look_up matrix 
def lookup_twl_wind(itime, check_lookup_file, TWL_matrix, swl, wSpd, wDir, icol):

    swl0 = swl
    swl  = min(13.2, swl)    # keep swl above the range to upper limit (this did not occur)  
    wSpd = min(23.0, wSpd)   # keep wind speed above the range to upper limit (this did not occur)  

    for irec, rec in enumerate(TWL_matrix):
        if(irec==0): continue #skip header
        swl_diff = abs(float(rec[1])-swl)
        spd_diff = abs(float(rec[2])-wSpd)
        dir_diff = abs(float(rec[3])-wDir)
        # match input swl, wspd, wdir centred around that in the lookup matrix within half of their resolution 
        if(swl_diff<=0.164042 and spd_diff<=1.0 and dir_diff<=5):
            flag = "TWL"
            twl = max(swl0, float(rec[icol]))
			# output for manual match 
            check_lookup_file.write(str(itime) + ": " + str(swl) + " , " + str(wSpd) + " , " + str(wDir) + " , " + str(twl) + '\n')
            return flag, twl

    # no match in the look up table
    # if swl is below the range, set twl and flag to null
    if(swl0<8.21):
        twl = "null"
        flag = "null"   
	# if only wind speed is below the range, set twl to swl and flag to "SWL". this maintains high water level with low wind speed 
    else:
        twl = swl0
        flag = "SWL"  
    return flag, twl


# main 	
meter_to_feet = 3.28084

base_dir = os.getcwd()

# need to create this folder to run
twl_dir = base_dir + os.sep + "TWL_Time_Series"

# directory to time series of adcirc water level for each transect
adcirc_wl_dir = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\ADCIRC_WL"

# directory to runup results
runup_dir = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Nearshore Hydraulics\Runup\JDF_WIND_022515"

# load transect information 
tsct_info_file = 'Clallam_TSCT_JuanDeFuca_AdcircNodes_New.txt'
tsct_info = numpy.loadtxt(tsct_info_file)

# load time series of wind 
wind_spd_file = "wind_spd.csv"
wind_spd = numpy.loadtxt(wind_spd_file, delimiter=",")
wind_dir_file = "wind_dir.csv"
wind_dir = numpy.loadtxt(wind_dir_file, delimiter=",")

os.chdir(runup_dir)
# load TWL lookup matrix 
TWL_matrix_file = "TWL.csv"
TWL_matrix = numpy.genfromtxt(TWL_matrix_file, dtype=None, delimiter=",")
# find the transect col in the TWL lookup matrix 
tsct_map2matrix = match_tsctid(TWL_matrix[0], tsct_info)

os.chdir(base_dir)
# open files to save maximum TWL for each storm and transect
twl_max_filename = "TWL_max_Wind" + ".csv"
twl_max_file = open(twl_max_filename, 'wb')
twl_max_writer = csv.writer(twl_max_file) 
# open files to save maximum SWL for each storm and transect
swl_max_filename = "SWL_max"  + ".csv"
swl_max_file = open(swl_max_filename, 'wb')
swl_max_writer = csv.writer(swl_max_file) 


# this was used to save file for checking look up manually 
check_lookup_filename = "TWL_Lookup_List.txt"
check_lookup_file = open(check_lookup_filename, 'wb')

# this was used to save data above the forcing matrix, no occurrence
above_range_filename = "above_range.txt"
above_range_file = open(above_range_filename, 'wb')

# loop through each transect
os.chdir(adcirc_wl_dir)
for itsct, tsct in enumerate(tsct_info):
    #if(itsct!=0): continue

    # get tsct_id and the corresponding col in lookup matrix 
    tsct_id = int(tsct[0])
    tsct_col = tsct_map2matrix[itsct]
    print "transect id:", tsct_id, tsct_col

    # open file to save time series of TWL for this transect
    os.chdir(twl_dir)
    twl_filename = "TSCT" + str(tsct_id).rjust(4,'0') + "_TWL"  + ".csv"
    twl_file = open(twl_filename, 'wb')
    twl_writer = csv.writer(twl_file)    
    # open file to save flag information(null, swl, twl) for this transect. what does the data in time series of TWL corresponds to
    twl_flag_filename = "TSCT" + str(tsct_id).rjust(4,'0') + "_TWL_Flag"  + ".csv"
    twl_flag_file = open(twl_flag_filename, 'wb')
    twl_flag_writer = csv.writer(twl_flag_file)  
    
    # load time series of adcirc water level for this transect
    os.chdir(adcirc_wl_dir)    
    swl_all_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_WL"  + ".csv"
    swl_all = numpy.loadtxt(swl_all_file, delimiter=",")
    
    # append tsct_id to first data of max twl
    SWL_max = []
    SWL_max.append("TSCT_" + str(tsct_id).rjust(4,'0'))    
    TWL_max = []
    TWL_max.append("TSCT_" + str(tsct_id).rjust(4,'0'))
    
	
    # loop through storm events
    for istorm in range(len(swl_all)):
        storm_id = swl_all[istorm][0]
        #if(storm_id!=39): continue 
	
	# check if storm id matches in swl, wind_spd and wind_dir csv file 
        print "storm:", swl_all[istorm][0], wind_spd[istorm][0], wind_spd[istorm][0]
        check_lookup_file.write("TSCT:" + str(tsct_id) + " Storm:" + str(storm_id)  + '\n')
		
	# append storm_id to first data of time series of twl
        TWL = []
        TWL.append(storm_id)          
        TWL_flag = []
        TWL_flag.append(storm_id)   

	# loop through time series of this storm starting after 2 days 
        for itime in range(49,193):
            swl  = float(swl_all[istorm][itime])*meter_to_feet
            wSpd = float(wind_spd[istorm][itime])
            wDir = float(wind_dir[istorm][itime])
            
            # find the corresponding twl in TWL lookup matrix 
            flag, twl_found = lookup_twl_wind(itime, check_lookup_file, TWL_matrix, swl, wSpd, wDir, tsct_col)
            TWL.append(twl_found)
            TWL_flag.append(flag)
            #print swl, wSpd, wDir
            
            # write data that above the range, not observed 
            if(swl>13.2):
                above_range_file.write("TSCT: " + str(tsct_id) + ", Storm: " + str(storm_id) + ", Time: " + str(itime) + ", SWL: " + str(swl)  + '\n')
            if(wSpd>23):
                above_range_file.write("TSCT: " + str(tsct_id) + ", Storm: " + str(storm_id) + ", Time: " + str(itime) + ", wSpd: " + str(wSpd)  + '\n')

        # write time series and get the maximum TWL of this storm for this transect 
        twl_writer.writerow(TWL)
        twl_flag_writer.writerow(TWL_flag)
        TWL_max.append(max([tw for tw in TWL[1:] if tw!="null"]))
        SWL_max.append(max([sw*meter_to_feet for sw in swl_all[istorm][1:] if sw!="null"]))

    twl_file.close()
    
    # write the maximum TWL for this transect 
    twl_max_writer.writerow(TWL_max)
    swl_max_writer.writerow(SWL_max)

# close all files 	
check_lookup_file.close()
above_range_file.close()    
swl_max_file.close()   
twl_max_file.close()    
        
os.chdir(base_dir)
