import os, csv
import numpy
from datetime import datetime

# main 
base_dir = os.getcwd()

# directory to adcirc runs 
adcirc_dir = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Puget_Sound_ADCIRC_Results-From_ATKINS\ADCIRC_Model_Runs"
# directory to adcirc wind data (for event list file)
adcirc_wind_dir = adcirc_dir + "\Model Runs - ADCIRC - Event List and BCs\ADCIRC_BCs_Wind" # get event list
# directory to folder used to save extracted time series of wl 
wl_dir = base_dir + os.sep + "ADCIRC_WL"  # folder to save wl time series for all tsct

# transect information file containing tsct_id and the corresponding adcirc node
tsct_info = numpy.loadtxt('Clallam_TSCT_JuanDeFuca_AdcircNodes_New.txt')
# sort transects with ascending order of adcirc nodes so that wl is obtained as the order adcirc output for optimization
tsct_info = tsct_info[tsct_info[:,1].argsort()]

# open csv writer for each transect 
tsct_data = [] 
csvfile = []
csvwriter = []
os.chdir(wl_dir)
for itsct, row in enumerate(tsct_info):
    tsct_id = int(row[0])
    node_id = int(row[1])    
    tsct_data.append([])
    filename = "TSCT" + str(tsct_id).rjust(4,'0') + "_WL"  + ".csv"
    csvfile.append(open(filename, 'wb'))
    csvwriter.append(csv.writer(csvfile[itsct]))
    print "TSCT_ID: " + str(tsct_id) + ", Node: " + str(node_id)
    
# get date of the event list as in python datetime
os.chdir(adcirc_wind_dir)
event_list = numpy.genfromtxt('EventList.txt', dtype=None)
event_dates = []
for i, event in enumerate(event_list):
    string_date = event[0] + ' ' + event[1]
    event_dates.append(datetime.strptime(string_date, "%d-%b-%Y %H:%M:%S"))
    
# loop through all 150 storm events 
post_date = datetime.strptime("31-Dec-1978 23:00:00", "%d-%b-%Y %H:%M:%S")
for storm_num in range(1,151):
    
    # ignore storm events before post_date
    tdiff = (event_dates[storm_num-1]-post_date).total_seconds()
    if(tdiff<=0): continue

    print "storm:", storm_num, event_dates[storm_num-1]
    
    # read adcirc output for the corresponding event
    dir_name = 'Prod_Run_Event_' + str(storm_num).rjust(3,'0')
    dir_path = adcirc_dir + os.sep + dir_name
    os.chdir(dir_path)
    fort63_file = open('fort.63', 'r')
    fort63_data = fort63_file.readlines()
    fort63_data.pop(0)
    fort63_data.pop(0)

    # for each transect, append the first data as the storm num
    for itsct, row in enumerate(tsct_info):  
        tsct_data[itsct] = [] 
        tsct_data[itsct].append(storm_num)

    # loop through adcirc output 
    for itmp, line in enumerate(fort63_data):
        # get current adcirc node
        try:    
            node = int(line.split()[0])
            # reset to first transect after each time step
            if(node==1):
                itsct = 0
                node_id = int(tsct_info[0][1])
        except:
            continue

        # obtain adcirc wl by matching adcirc node of the transect to current adcirc node
        if(node==node_id and itsct<=len(tsct_info)-1):
            tsct_data[itsct].append(float(line.split()[1]))
            itsct = itsct + 1
            # to account for different transects have the same adcirc node 
            for i in range(len(tsct_info)-itsct):
                if(int(tsct_info[itsct][1])==node_id):
                    tsct_data[itsct].append(float(line.split()[1]))
                    itsct = itsct + 1
                else:
                    break
            # obtain next non_repeat adcirc node from the tsct list	
            if(itsct<=len(tsct_info)-1):
                node_id = int(tsct_info[itsct][1])

    # output time series of this storm for each transect
    for itsct, row in enumerate(tsct_info):
        csvwriter[itsct].writerow(tsct_data[itsct])

os.chdir(base_dir)

# close csv files for all transect
for itsct, row in enumerate(tsct_info):
    csvfile[itsct].close()
