import os, csv
import math, numpy

# main 
meter_to_feet = 3.28084
base_dir = os.getcwd()

dwl_dir = base_dir + os.sep + "DWL_Time_Series_Merged"

twl_dir = base_dir + os.sep + "TWL_Time_Series_Merged"

# directory to time series of adcirc wl
adcirc_swl_dir = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\ADCIRC_WL"

# directory to time series of DWL from wind
wind_dwl_dir   = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\Wind\DWL_Time_Series"
# directory to time series of DWl from swell
swell_dwl_dir  = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\Swell\DWL_Time_Series"

# directory to time series of TWL from wind
wind_twl_dir   = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\Wind\TWL_Time_Series"
# directory to time series of TWL from swell
swell_twl_dir  = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Juan_de_Fuca\Swell\TWL_Time_Series"

# transect information file containing tsct_id and the corresponding adcirc node
tsct_info = numpy.loadtxt('Clallam_TSCT_JuanDeFuca_AdcircNodes_New.txt')

# open files to save maximum DWL for each storm and transect
dwl_max_filename = "DWL_max_merged"  + ".csv"
dwl_max_file = open(dwl_max_filename, 'wb')
dwl_max_writer = csv.writer(dwl_max_file) 

# open files to save maximum TWL for each storm and transect
twl_max_filename = "TWL_max_merged"  + ".csv"
twl_max_file = open(twl_max_filename, 'wb')
twl_max_writer = csv.writer(twl_max_file) 

for itsct, tsct in enumerate(tsct_info):
    #if(itsct!=0): continue

    # get tsct_id and the corresponding col in lookup matrix 
    tsct_id = int(tsct[0])
    print "transect id:", tsct_id
    
    # load time series of adcirc water level for this transect
    os.chdir(adcirc_swl_dir)    
    adcirc_swl_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_WL"  + ".csv"
    adcirc_swl = numpy.loadtxt(adcirc_swl_file, delimiter=",")

    # load time series of wind dwl for this transect
    os.chdir(wind_dwl_dir)    
    wind_dwl_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_DWL"  + ".csv"
    wind_dwl = numpy.genfromtxt(wind_dwl_file, dtype=None, delimiter=",")     
    
    # load time series of swell dwl for this transect
    os.chdir(swell_dwl_dir)    
    swell_dwl_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_DWL"  + ".csv"
    swell_dwl = numpy.genfromtxt(swell_dwl_file, dtype=None, delimiter=",")     
            
    # load time series of wind twl for this transect
    os.chdir(wind_twl_dir)    
    wind_twl_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_TWL"  + ".csv"
    wind_twl = numpy.genfromtxt(wind_twl_file, dtype=None, delimiter=",")     
    
    # load time series of swell twl for this transect
    os.chdir(swell_twl_dir)    
    swell_twl_file = "TSCT" + str(tsct_id).rjust(4,'0') + "_TWL"  + ".csv"
    swell_twl = numpy.genfromtxt(swell_twl_file, dtype=None, delimiter=",") 
   
    # append tsct_id to first data of max twl
    DWL_max = []
    DWL_max.append("TSCT_" + str(tsct_id).rjust(4,'0'))    
          
    # append tsct_id to first data of max twl
    TWL_max = []
    TWL_max.append("TSCT_" + str(tsct_id).rjust(4,'0'))    

    os.chdir(dwl_dir)
    dwl_filename = "TSCT" + str(tsct_id).rjust(4,'0') + "_DWL_merged"  + ".csv"
    dwl_file = open(dwl_filename, 'wb')
    dwl_writer = csv.writer(dwl_file) 

    os.chdir(twl_dir)
    twl_filename = "TSCT" + str(tsct_id).rjust(4,'0') + "_TWL_merged"  + ".csv"
    twl_file = open(twl_filename, 'wb')
    twl_writer = csv.writer(twl_file) 

    # loop through storm events
    for istorm in range(len(adcirc_swl)):
        
        storm_id = adcirc_swl[istorm][0]
	# check if storm id match 
        print "storm:", adcirc_swl[istorm][0], wind_twl[istorm][0], swell_twl[istorm][0]
        
	# append storm_id to first data of time series of dwl
        DWL = [] 
        DWL.append(storm_id)     
        # append storm_id to first data of time series of twl
        TWL = [] 
        TWL.append(storm_id)  

	# loop through time series of this storm starting after 2 days 
        for itime in range(1,145):
            swl = float(adcirc_swl[istorm][itime+48])*meter_to_feet
            # DWL
            if(wind_dwl[istorm][itime]!="null"):
                dwl_w = float(wind_dwl[istorm][itime])
            else:
                dwl_w = swl
            if(swell_dwl[istorm][itime]!="null"):
                dwl_s = float(swell_dwl[istorm][itime])
            else:
                dwl_s = swl
            print swl, dwl_w-swl, dwl_s-swl
            dynwl = math.sqrt((dwl_w-swl)**2.0 + (dwl_s-swl)**2.0)
            dwl_m = swl + dynwl
            if(wind_dwl[istorm][itime]=="null" and swell_dwl[istorm][itime]=="null"): dwl_m='null'
            DWL.append(dwl_m)
            
            # TWL 
            if(wind_twl[istorm][itime]!="null"):
                twl_w = float(wind_twl[istorm][itime])
            else:
                twl_w = swl
            if(swell_twl[istorm][itime]!="null"):
                twl_s = float(swell_twl[istorm][itime])
            else:
                twl_s = swl
            print swl, twl_w-swl, twl_s-swl
            runup = math.sqrt((twl_w-swl)**2.0 + (twl_s-swl)**2.0)
            twl_m = swl + runup 
            if(wind_twl[istorm][itime]=="null" and swell_twl[istorm][itime]=="null"): twl_m='null'
            TWL.append(twl_m)
        
        # write time series of DWL and TWL
        dwl_writer.writerow(DWL)
        twl_writer.writerow(TWL)      
             
        DWL_max.append(max([tw for tw in DWL[1:] if tw!='null']))           
        TWL_max.append(max([tw for tw in TWL[1:] if tw!='null']))
     
    dwl_max_writer.writerow(DWL_max)
    twl_max_writer.writerow(TWL_max)

    dwl_file.close()           
    twl_file.close()
        
dwl_max_file.close()    
twl_max_file.close()
os.chdir(base_dir)
    