clear all;
close all;
warning('off','MATLAB:nearlySingularMatrix');

meter_to_feet = 3.28084;

% import transect information
tsct_file = 'POC_SELFE_Nodes';
[selfe_id,~,tsct_info] = xlsread(['./' tsct_file '.xlsx']);
[nTsct, dummy] = size(selfe_id);

year_str = 1980;
year_end = 2010;

tsct_elev = {};
for itsct=1:nTsct
    tsct_elev{itsct} = [];
end
for iy=year_str:year_end
    disp(['Processing Year: ', num2str(iy)])
    % directory to SELFE data
    if iy<=1991
       selfe_path = 'F:/CMOP_WSE/';
    else
       selfe_path = 'E:/CMOP_WSE/'; 
    end

    file_path  = strcat(selfe_path, num2str(iy), '/run/');
    nfiles = numel(dir([file_path,'*.nc']));
    
    date_day01 = datenum([iy 1 1]);    
    for jf=1:nfiles
        fname=strcat(file_path,num2str(jf),'_elev.61.nc');
        % read elevation data
        try
            elev_all = ncread(fname,'elev');
            % convert the seconds to days, 
            % 0.1s is added to prevent float number round 
            time_pst = date_day01+double(ncread(fname,'time')+0.1)/86400.0; 
            time_str = cellstr(datestr(time_pst',30));
            % reduce to 1 hr interval & concat data 
            elev_all = elev_all(selfe_id,4:4:end)*meter_to_feet;
            time_pst = time_pst(4:4:end);
            time_str = time_str(4:4:end);
        catch
            disp(['data is missing for day: ', num2str(jf)]);
            continue;
        end
        for itsct=1:nTsct
            for it=1:24
                if(elev_all(itsct, it)>=8.2102) % lower limit to 8.2102
                    swl = elev_all(itsct, it);
                    tsct_elev{itsct} = [tsct_elev{itsct}; [time_str(it) num2str(swl)]];
                end
            end
        end
        
    end % loop files, one day per file 
end % loop years

% output file path
out_path = 'C:/Users/aachen/Desktop/New folder/Pacific_Ocean/SELFE_SWL/';
for itsct=1:nTsct
    xlsfile = strcat(out_path, 'TSCT_', tsct_info{itsct+1,1}, '.xlsx');
    xlswrite(xlsfile, tsct_elev{itsct})
end 

