import os, time
import csv, numpy
from datetime import datetime
import pyexcel
import pyexcel_xlsx

# find col of each transect in TWL_look_up matrix 
def match_tsctid(TWL_matrix_header, tsct_info):

    tsct_map = []

    for tsct in tsct_info:
        tsct_id = int(tsct[0][3:])
        for irec, rec in enumerate(TWL_matrix_header):
            if irec<=4: continue
            tsct_id_in_matrix = rec.split("_")[1].lstrip('0')
            if(tsct_id==int(tsct_id_in_matrix)):
                tsct_map.append(irec)
                break
                
    return tsct_map
            
# look up TWL in TWL_look_up matrix 
def lookup_swell(forcing_matrix, swl, whs, wtp, wdr):

    swl  = min(13.2, swl)    # keep swl above the range to upper limit (this did not occur)  
    whs  = min(9.5,  whs)    # keep wave height above the range to upper limit (this did not occur)  
    wtp  = min(23.0, wtp)    # keep wave period above the range to upper limit (few occurence)

    irow = -1
    for irec, rec in enumerate(forcing_matrix):
        if(irec==0): continue #skip header
        swl_diff = abs(float(rec[1])-swl)
        whs_diff = abs(float(rec[2])-whs)
        wtp_diff = abs(float(rec[3])-wtp)
        wdr_diff = abs(float(rec[4])-wdr)
	# match input swl, wspd, wdir centred around that in the lookup matrix within half of their resolution 
        if(swl_diff<=0.164042 and whs_diff<=0.5 and wtp_diff<=1 and wdr_diff<=7.5):
            return irec
 
    return irow


# main 	
meter_to_feet = 3.28084
base_dir = os.getcwd()

# need to create this folder to save the results 
ts_dir = base_dir + os.sep + "Time_Series"

# directory to time series of selfe water level for each transect
selfe_wl_dir = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Statistics\Pacific_Ocean\SELFE_SWL"

# directory to runup results
runup_dir = r"U:\20281310073\Work Tasks\3_Clallam_Coastal_WA_PMR\Coastal Engineering\Nearshore Hydraulics\Runup\OCEAN\_WaveRunUp_Outputs\csv"

# load transect information 
tsct_info_file = 'Clallam_SELFE_Nodes.xlsx'
tsct_book = pyexcel.load_book(tsct_info_file)
tsct_info = tsct_book['Nodes'].to_array()[1:]
print tsct_info

# load time series of wave 
wis_data_file = "ST83004_v01.onlns"
wis_data = numpy.loadtxt(wis_data_file, usecols=(0, 25, 26, 31))

os.chdir(runup_dir)
# load DWL lookup matrix 
DWL_matrix_file = "RefWL.csv"
DWL_matrix = numpy.genfromtxt(DWL_matrix_file, dtype=None, delimiter=",")

# load TWL lookup matrix 
TWL_matrix_file = "TWL.csv"
TWL_matrix = numpy.genfromtxt(TWL_matrix_file, dtype=None, delimiter=",")

# load Irriberran number flag lookup matrix
IRI_matrix_file = "IRI_Flag.csv"
IRI_matrix = numpy.genfromtxt(IRI_matrix_file, dtype=None, delimiter=",")

# find the transect col in the TWL lookup matrix 
tsct_map2matrix = match_tsctid(TWL_matrix[0], tsct_info)

# this was used to save file for checking look up manually
check_lookup_filename = "TWL_Lookup_List.txt"
check_lookup_file = open(check_lookup_filename, 'wb')

# this was used to save swell above the forcing matrix, no occurrence
above_range_filename = "above_range.txt"
above_range_file = open(above_range_filename, 'wb')

# loop through each transect
nwis = len(wis_data)-1
for itsct, tsct in enumerate(tsct_info):
    
    #if(itsct!=0): continue

    # get tsct_id and the corresponding col in lookup matrix 
    tsct_id = tsct[0]
    tsct_col = tsct_map2matrix[itsct]

    print "transect id:", tsct_id, tsct_col

    # open file to save time series of this transect
    os.chdir(ts_dir)
    ts_filename = "TSCT_" + tsct_id + "_TS"  + ".csv"
    ts_file = open(ts_filename, 'wb')
    ts_writer = csv.writer(ts_file)
    ts_writer.writerow(['time', 'SWL', 'Hs', 'Tp', 'Dir', 'DWL', 'TWL', 'Flag'])
    
    # load time series of selfe water level for this transect
    os.chdir(selfe_wl_dir)    
    swl_file = 'TSCT_' + tsct_id  + ".xlsx"
    swl_book = pyexcel.load_book(swl_file)
    swl_data = swl_book['Sheet1'].to_array()
    
    # loop through time series of swell
    iwis = -1  # initialize counter in wis_data
    for irec, rec in enumerate(swl_data):
        
        time_str = rec[0]
        time_pst = datetime.strptime(rec[0].replace('T',''), "%Y%m%d%H%M%S")
        
        for it in range(iwis+1,nwis):
            string_date = str(int(wis_data[it][0]))
            time_in_wis = datetime.strptime(string_date, "%Y%m%d%H%M%S")
            # change GMT/UTC to PST time zone for WIS data
            time_diff   = (time_in_wis-time_pst).total_seconds()/3600 - 8
            if(time_diff==0.0):
                iwis = it
                break

        swl = float(rec[1])
        whs = float(wis_data[iwis][1])
        wtp = float(wis_data[iwis][2])
        wdr = float(wis_data[iwis][3])
        
	# find the corresponding twl in TWL lookup matrix 
        irow_found = lookup_swell(TWL_matrix, swl, whs, wtp, wdr)

        if(irow_found!=-1):
            dwl_found = max(swl,float(DWL_matrix[irow_found][tsct_col]))
            twl_found = max(swl,float(TWL_matrix[irow_found][tsct_col]))
            flag = "TWL" + IRI_matrix[irow_found][tsct_col]
            check_lookup_file.write(time_str + ": " + str(swl) + " , " + str(whs) + " , " + str(wtp) + " , " + str(wdr) + " , " + str(twl_found) + " , " + flag + '\n')
        else:
            if(swl<8.21 or numpy.isnan(whs)):
                dwl_found = "null"
                twl_found = "null"
                flag = "null"
            # if only wave is out of the range, set twl to swl and flag to "SWL". this maintains high water level with low wave height 
            else:
                dwl_found = swl
                twl_found = swl
                flag = "SWL"  

        # print time_str, string_date, swl, whs, wtp, wdr,twl_found, flag
        # time.sleep(1)

	# write data that above the range, not observed 
        if(swl>13.2):
            above_range_file.write("TSCT: " + str(tsct_id) + ", Time: " + time_str + ", SWL: " + str(swl)  + '\n')
        if(whs>9.5):
            above_range_file.write("TSCT: " + str(tsct_id) + ", Time: " + time_str + ", whs: " + str(whs)  + '\n')
        if(wtp>23.0):
            above_range_file.write("TSCT: " + str(tsct_id) + ", Time: " + time_str + ", wtp: " + str(wtp)  + '\n')

	# write time series of SWL, DWL, TWL and Flag
        ts_writer.writerow([time_str, swl, whs, wtp, wdr, dwl_found, twl_found, flag])

    ts_file.close()

os.chdir(base_dir)
    
# close all files 	
check_lookup_file.close()
above_range_file.close()    
